@@ -6,24 +6,24 @@
 
 #' Export an H2O Data Frame to a File
 #'
-#' Exports an \linkS4class{H2OFrame} (which can be either VA or FV) to a file.
+#' Exports an H2O Frame (which can be either VA or FV) to a file.
 #' This file may be on the H2O instace's local filesystem, or to HDFS (preface
 #' the path with hdfs://) or to S3N (preface the path with s3n://).
 #'
 #' In the case of existing files \code{forse = TRUE} will overwrite the file.
 #' Otherwise, the operation will fail.
 #'
-#' @param data An \linkS4class{H2OFrame} data frame.
+#' @param data An H2O Frame data frame.
 #' @param path The path to write the file to. Must include the directory and
 #'        filename. May be prefaced with hdfs:// or s3n://. Each row of data
 #'        appears as line of the file.
 #' @param force logical, indicates how to deal with files that already exist.
 #' @examples
-#'\donttest{
+#'\dontrun{
 #' library(h2o)
-#' localH2O <- h2o.init()
+#' h2o.init()
 #' irisPath <- system.file("extdata", "iris.csv", package = "h2o")
-#' iris.hex <- h2o.uploadFile(localH2O, path = irisPath)
+#' iris.hex <- h2o.uploadFile(path = irisPath)
 #'
 #' # These aren't real paths
 #' # h2o.exportFile(iris.hex, path = "/path/on/h2o/server/filesystem/iris.csv")
@@ -32,45 +32,45 @@
 #' }
 #' @export
 h2o.exportFile <- function(data, path, force = FALSE) {
-  if (!is(data, "H2OFrame"))
-    stop("`data` must be an H2OFrame object")
+  if (!is.Frame(data))
+    stop("`data` must be an H2O Frame object")
 
   if(!is.character(path) || length(path) != 1L || is.na(path) || !nzchar(path))
     stop("`path` must be a non-empty character string")
 
   if(!is.logical(force) || length(force) != 1L || is.na(force))
     stop("`force` must be TRUE or FALSE")
-  res <- .h2o.__remoteSend(data@conn, .h2o.__EXPORT_FILES(data,path,force))
-  .h2o.__waitOnJob(data@conn, res$job$key$name)
+
+  .h2o.__remoteSend(.h2o.__EXPORT_FILES(data,path,force))
 }
 
 #'
-#' Export a H2OFrame to HDFS
+#' Export a Model to HDFS
 #'
-#' Exports an \linkS4class{H2OFrame} to HDFS.
+#' Exports an \linkS4class{H2OModel} to HDFS.
 #'
-#' @param data an \linkS4class{H2OFrame} class object.
-#' @param path The path to write the data to. Must include the driectory and
+#' @param object an \linkS4class{H2OModel} class object.
+#' @param path The path to write the model to. Must include the driectory and
 #'        filename.
 #' @param force logical, indicates how to deal with files that already exist.
 #' @export
-h2o.exportHDFS <- function(data,path,force=FALSE) { h2o.exportFile(data,path,force) }
+h2o.exportHDFS <- function(object, path, force=FALSE) { h2o.exportFile(object,path,force) }
 
 #' Download H2O Data to Disk
 #'
 #' Download an H2O data set to a CSV file on the local disk
 #'
 #' @section Warning: Files located on the H2O server may be very large! Make
-#'        sure you have enough hard drive psace to accomoadet the entire file.
-#' @param data an \linkS4class{H2OFrame} object to be downloaded.
+#'        sure you have enough hard drive space to accomodate the entire file.
+#' @param data an H2O Frame object to be downloaded.
 #' @param filename A string indicating the name that the CSV file should be
 #'        should be saved to.
 #' @examples
 #' \donttest{
 #' library(h2o)
-#' localH2O <- h2o.init()
+#' h2o.init()
 #' irisPath <- system.file("extdata", "iris_wheader.csv", package = "h2o")
-#' iris.hex <- h2o.uploadFile(localH2O, path = irisPath)
+#' iris.hex <- h2o.uploadFile(path = irisPath)
 #'
 #' myFile <- paste(getwd(), "my_iris_file.csv", sep = .Platform$file.sep)
 #' h2o.downloadCSV(iris.hex, myFile)
@@ -79,10 +79,11 @@ h2o.exportHDFS <- function(data,path,force=FALSE) { h2o.exportFile(data,path,for
 #' }
 #' @export
 h2o.downloadCSV <- function(data, filename) {
-  if (!is(data, "H2OFrame"))
-    stop("`data` must be an H2OFrame object")
+  if (!is.Frame(data))
+    stop("`data` must be an H2O Frame object")
 
-  str <- paste0('http://', data@conn@ip, ':', data@conn@port, '/3/DownloadDataset?frame_id=', data@frame_id)
+  conn = h2o.getConnection()
+  str <- paste0('http://', conn@ip, ':', conn@port, '/3/DownloadDataset?frame_id=', h2o.getId(data))
   has_wget <- nzchar(Sys.which('wget'))
   has_curl <- nzchar(Sys.which('curl'))
   if(!(has_wget || has_curl))
@@ -116,10 +117,10 @@ h2o.downloadCSV <- function(data, filename) {
 #' @param force logical, indicates how to deal with files that already exist.
 #' @seealso \code{\link{h2o.loadModel}} for loading a model to H2O from disk
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # library(h2o)
-#' # localH2O <- h2o.init()
-#' # prostate.hex <- h2o.importFile(localH2O, path = paste("https://raw.github.com",
+#' # h2o.init()
+#' # prostate.hex <- h2o.importFile(path = paste("https://raw.github.com",
 #' #   "h2oai/h2o-2/master/smalldata/logreg/prostate.csv", sep = "/"),
 #' #   destination_frame = "prostate.hex")
 #' # prostate.glm <- h2o.glm(y = "CAPSULE", x = c("AGE","RACE","PSA","DCAPS"),
@@ -131,8 +132,7 @@ h2o.saveModel <- function(object, path="", force=FALSE) {
   if(!is(object, "H2OModel")) stop("`object` must be an H2OModel object")
   if(!is.character(path) || length(path) != 1L || is.na(path)) stop("`path` must be a character string")
   if(!is.logical(force) || length(force) != 1L || is.na(force)) stop("`force` must be TRUE or FALSE")
-  force <- as.integer(force)
   path <- file.path(path, object@model_id)
-  res <- .h2o.__remoteSend(object@conn, paste0("Models.bin/",object@model_id),dir=path,force=force,h2oRestApiVersion=99)
+  res <- .h2o.__remoteSend(paste0("Models.bin/",object@model_id),dir=path,force=force,h2oRestApiVersion=99)
   res$dir
 }
