@@ -4,7 +4,7 @@
 #' Performs k-means clustering on an H2O dataset.
 #'
 #'
-#' @param training_frame An \linkS4class{H2OFrame} object containing the
+#' @param training_frame An H2O Frame object containing the
 #'        variables in the model.
 #' @param x (Optional) A vector containing the data columns on
 #'        which k-means operates.
@@ -23,9 +23,9 @@
 #'        centers. Possible values are "Random": for random initialization,
 #'        "PlusPlus": for k-means plus initialization, or "Furthest": for
 #'        initialization at the furthest point from each successive center.
-#'        Additionally, the user may specify a the initial centers as a
-#'        matrix, data.frame, H2OFrame, or list of vectors. For matrices,
-#'        data.frames, and H2OFrames, each row of the respective structure
+#'        Additionally, the user may specify a the initial centers as a matrix,
+#'        data.frame, Frame, or list of vectors. For matrices,
+#'        data.frames, and Frames, each row of the respective structure
 #'        is an initial center. For lists of vectors, each vector is an
 #'        initial center.
 #' @param seed (Optional) Random seed used to initialize the cluster centroids.
@@ -39,11 +39,13 @@
 #'          \code{\link{h2o.betweenss}}, \code{\link{h2o.tot_withinss}}, \code{\link{h2o.withinss}},
 #'          \code{\link{h2o.centersSTD}}, \code{\link{h2o.centers}}
 #' @examples
+#' \donttest{
 #' library(h2o)
-#' localH2O <- h2o.init()
+#' h2o.init()
 #' prosPath <- system.file("extdata", "prostate.csv", package="h2o")
-#' prostate.hex <- h2o.uploadFile(localH2O, path = prosPath)
+#' prostate.hex <- h2o.uploadFile(path = prosPath)
 #' h2o.kmeans(training_frame = prostate.hex, k = 10, x = c("AGE", "RACE", "VOL", "GLEASON"))
+#' }
 #' @export
 h2o.kmeans <- function(training_frame, x, k,
                        model_id,
@@ -56,17 +58,17 @@ h2o.kmeans <- function(training_frame, x, k,
                        fold_assignment = c("AUTO","Random","Modulo"),
                        keep_cross_validation_predictions = FALSE)
 {
-  # Training_frame may be a key or an H2OFrame object
-  if (!inherits(training_frame, "H2OFrame"))
+  # Training_frame may be a key or an H2O Frame object
+  if( !is.Frame(training_frame) )
     tryCatch(training_frame <- h2o.getFrame(training_frame),
              error = function(err) {
-               stop("argument \"training_frame\" must be a valid H2OFrame or key")
+               stop("argument \"training_frame\" must be a valid Frame or key")
              })
 
   # Gather user input
   parms <- list()
   if( !(missing(x)) )
-      parms$ignored_columns <- .verify_datacols(training_frame, x)$cols_ignore
+    parms$ignored_columns <- .verify_datacols(training_frame, x)$cols_ignore
   if(!missing(k))
     parms$k <- k
   parms$training_frame <- training_frame
@@ -87,12 +89,12 @@ h2o.kmeans <- function(training_frame, x, k,
   if( !missing(keep_cross_validation_predictions) )  parms$keep_cross_validation_predictions  <- keep_cross_validation_predictions
 
   # Check if init is an acceptable set of user-specified starting points
-  if( is.data.frame(init) || is.matrix(init) || is.list(init) || inherits(init, "H2OFrame") ) {
+  if( is.data.frame(init) || is.matrix(init) || is.list(init) || is.Frame(init) ) {
     parms[["init"]] <- "User"
-    # Convert user-specified starting points to H2OFrame
+    # Convert user-specified starting points to Frame
     if( is.data.frame(init) || is.matrix(init) || is.list(init) ) {
       if( !is.data.frame(init) && !is.matrix(init) ) init <- t(as.data.frame(init))
-      init <- as.h2o(init, training_frame@conn)
+      init <- as.h2o(init)
     }
     parms[["user_points"]] <- init
     # Set k
@@ -109,5 +111,5 @@ h2o.kmeans <- function(training_frame, x, k,
   }
 
   # Error check and build model
-  .h2o.createModel(training_frame@conn, 'kmeans', parms)
+  .h2o.modelJob('kmeans', parms)
 }
