@@ -5,15 +5,13 @@
 # ------------------------------- Helper Functions --------------------------- #
 # Used to verify data, x, y and turn into the appropriate things
 .verify_dataxy <- function(data, x, y, autoencoder = FALSE) {
-  if(!is(data,  "H2OFrame"))
-    stop('`data` must be an H2OFrame object')
   if(!is.character(x) && !is.numeric(x))
     stop('`x` must be column names or indices')
   if( !autoencoder )
     if(!is.character(y) && !is.numeric(y))
       stop('`y` must be a column name or index')
 
-  cc <- colnames(data)
+  cc <- colnames(chk.Frame(data))
 
   if(is.character(x)) {
     if(!all(x %in% cc))
@@ -54,12 +52,10 @@
 }
 
 .verify_datacols <- function(data, cols) {
-  if(!is(data, "H2OFrame"))
-    stop('`data` must be an H2OFrame object')
   if(!is.character(cols) && !is.numeric(cols))
     stop('`cols` must be column names or indices')
 
-  cc <- colnames(data)
+  cc <- colnames(chk.Frame(data))
   if(length(cols) == 1L && cols == '')
     cols <- cc
   if(is.character(cols)) {
@@ -95,32 +91,41 @@
   cf_matrix
 }
 
-.h2o.startModelJob <- function(conn = h2o.getConnection(), algo, params, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
-  .key.validate(params$key_value)
+
+.h2o.modelJob <- function( algo, params, h2oRestApiVersion=.h2o.__REST_API_VERSION ) {
+  .eval.frame(params$training_frame)
+  if( !is.null(params$validation_frame) )
+    .eval.frame(params$validation_frame)
+  job <- .h2o.startModelJob(algo, params, h2oRestApiVersion)
+  h2o.getFutureModel(job)
+}
+
+.h2o.startModelJob <- function(algo, params, h2oRestApiVersion) {
+  .key.validate(params$key)
   #---------- Force evaluate temporary ASTs ----------#
-  ALL_PARAMS <- .h2o.getModelParameters(algo = algo)
+  ALL_PARAMS <- .h2o.__remoteSend(method = "GET", h2oRestApiVersion = h2oRestApiVersion, .h2o.__MODEL_BUILDERS(algo))$model_builders[[algo]]$parameters
 
   # R treats integer as not numeric: FIXME move into checkAndUnifyModelParameters
   params <- lapply(params, function(x) { if(is.integer(x)) x <- as.numeric(x); x })
   #---------- Check user parameter types ----------#
   param_values <- .h2o.checkAndUnifyModelParameters(algo = algo, allParams = ALL_PARAMS, params = params)
   #---------- Validate parameters ----------#
-  .h2o.validateModelParameters(conn, algo, param_values, h2oRestApiVersion)
+  .h2o.validateModelParameters(algo, param_values, h2oRestApiVersion)
   #---------- Build! ----------#
-  res <- .h2o.__remoteSend(conn, method = "POST", .h2o.__MODEL_BUILDERS(algo), .params = param_values, h2oRestApiVersion = h2oRestApiVersion)
+  res <- .h2o.__remoteSend(method = "POST", .h2o.__MODEL_BUILDERS(algo), .params = param_values, h2oRestApiVersion = h2oRestApiVersion)
 
   job_key  <- res$job$key$name
   dest_key <- res$job$dest$name
 
-  new("H2OModelFuture",conn=conn, job_key=job_key, model_id=dest_key)
+  new("H2OModelFuture",job_key=job_key, model_id=dest_key)
 }
 
 #
 # Validate given parameters against algorithm parameters validation
 # REST end-point. Stop execution in case of validation error.
 #
-.h2o.validateModelParameters <- function(conn = h2o.getConnection(), algo, params, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
-  validation <- .h2o.__remoteSend(conn, method = "POST", paste0(.h2o.__MODEL_BUILDERS(algo), "/parameters"), .params = params, h2oRestApiVersion = h2oRestApiVersion)
+.h2o.validateModelParameters <- function(algo, params, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
+  validation <- .h2o.__remoteSend(method = "POST", paste0(.h2o.__MODEL_BUILDERS(algo), "/parameters"), .params = params, h2oRestApiVersion = h2oRestApiVersion)
   if(length(validation$messages) != 0L) {
     error <- lapply(validation$messages, function(i) {
       if( i$message_type == "ERROR" )
@@ -137,38 +142,25 @@
   }
 }
 
-.h2o.createModel <- function(conn = h2o.getConnection(), algo, params, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
- params$training_frame <- get("training_frame", parent.frame())
- tmp_train <- !.is.eval(params$training_frame)
- if( tmp_train ) {
-    temp_train_key <- params$training_frame@frame_id
-    .h2o.eval.frame(conn = conn, ast = params$training_frame@mutable$ast, frame_id = temp_train_key)
- }
-
- if (!is.null(params$validation_frame)){
-    params$validation_frame <- get("validation_frame", parent.frame())
-    tmp_valid <- !.is.eval(params$validation_frame)
-    if( tmp_valid ) {
-      temp_valid_key <- params$validation_frame@frame_id
-      .h2o.eval.frame(conn = conn, ast = params$validation_frame@mutable$ast, frame_id = temp_valid_key)
-    }
-  }
-
-  h2o.getFutureModel(.h2o.startModelJob(conn, algo, params, h2oRestApiVersion))
+.h2o.createModel <- function(algo, params, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
+  h2o.getFutureModel(.h2o.startModelJob(algo, params, h2oRestApiVersion))
 }
 
+#' Get future model
+#'
+#' @rdname h2o.getFutureModel
+#' @param object H2OModel
+#' @export
 h2o.getFutureModel <- function(object) {
-  .h2o.__waitOnJob(object@conn, object@job_key)
-  h2o.getModel(object@model_id, object@conn)
+  .h2o.__waitOnJob(object@job_key)
+  h2o.getModel(object@model_id)
 }
 
 .h2o.prepareModelParameters <- function(algo, params, is_supervised) {
-  if (!is.null(params$training_frame)) {
-    params$training_frame <- .h2o.checkFrameParam(params$training_frame, "training_frame")
-  }
-  if (!is.null(params$validation_frame)) {
-    params$validation_frame <- .h2o.checkFrameParam(params$validation_frame, "validation_frame")
-  }
+  if (!is.null(params$training_frame))
+    params$training_frame <- chk.Frame(params$training_frame)
+  if (!is.null(params$validation_frame))
+    params$validation_frame <- chk.Frame(params$validation_frame)
 
   # Check if specified model request is for supervised algo
   isSupervised <- if (!is.null(is_supervised)) is_supervised else .isSupervised(algo, params)
@@ -194,9 +186,8 @@ h2o.getFutureModel <- function(object) {
   params
 }
 
-.h2o.getModelParameters <- function(conn = h2o.getConnection(), algo, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
-  allParameters <- .h2o.__remoteSend(conn, method = "GET", .h2o.__MODEL_BUILDERS(algo), h2oRestApiVersion = h2oRestApiVersion)$model_builders[[algo]]$parameters
-  allParameters
+.h2o.getModelParameters <- function(algo, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
+  .h2o.__remoteSend(method = "GET", .h2o.__MODEL_BUILDERS(algo), h2oRestApiVersion = h2oRestApiVersion)$model_builders[[algo]]$parameters
 }
 
 .h2o.checkAndUnifyModelParameters <- function(algo, allParams, params, hyper_params = list()) {
@@ -220,10 +211,8 @@ h2o.getFutureModel <- function(object) {
 
   #---------- Create parameter list to pass ----------#
   param_values <- lapply(params, function(i) {
-    if(is(i, "H2OFrame"))
-      i@frame_id
-    else
-      i
+    if(is.Frame(i))  h2o.getId(i)
+    else             i
   })
 
   param_values
@@ -286,9 +275,8 @@ h2o.getFutureModel <- function(object) {
         paramValue <- .collapse(paramValue)
     }
   }
-  if (is(paramValue, "H2OFrame")) {
-    paramValue <- paramValue@frame_id
-  }
+  if( is.Frame(paramValue) )
+    paramValue <- h2o.getId(paramValue)
   paramValue
 }
 
@@ -319,24 +307,26 @@ h2o.getFutureModel <- function(object) {
         }
         # If there is no error then transform hyper values
         if (!nzchar(e)) {
-          mapping <- .type.map[paramDef$type,]
-          type <- mapping[1L, 1L]
-          transf_hyper_vals <- sapply(hyper_vals,
-                                      simplify = !is.list(hyper_vals),
-                                      function(hv) {
-                                        # R does not treat integers as numeric
-                                        if (is.integer(hv)) {
-                                          hv <- as.numeric(hv)
-                                        }
-                                        # Force evaluation of frames and fetch frame_id as
-                                        # a side effect
-                                        if (type == "H2OFrame") {
-                                          hv <- .h2o.checkFrameParam(hv, name)@frame_id
-                                        }
-                                        # Returns transformed value
-                                        hv <- .h2o.transformParam(paramDef, hv, collapseArrays = FALSE)
-                                        hv
-                                      })
+          is_scalar <- .type.map[paramDef$type,][1L, 2L]
+          transf_fce <- function(hv) {
+                          # R does not treat integers as numeric
+                          if (is.integer(hv)) {
+                            hv <- as.numeric(hv)
+                          }
+                          mapping <- .type.map[paramDef$type,]
+                          type <- mapping[1L, 1L]
+                          # Note: we apply this transformatio also for types 
+                          # reported by the backend as scalar because of PUBDEV-1955
+                          if (is.list(hv)) {
+                            hv <- as.vector(hv, mode=type)
+                          }
+                          # Force evaluation of frames and fetch frame_id as
+                          # a side effect
+                          if (is.Frame(hv) )
+                            hv <- h2o.getId(hv)
+                          .h2o.transformParam(paramDef, hv, collapseArrays = FALSE)
+                        }
+          transf_hyper_vals <- if (is_scalar) sapply(hyper_vals,transf_fce) else lapply(hyper_vals, transf_fce) 
           hyper_params[[name]] <<- transf_hyper_vals
         }
       }
@@ -355,13 +345,16 @@ h2o.getFutureModel <- function(object) {
 #'
 #' This method dispatches on the type of H2O model to select the correct
 #' prediction/scoring algorithm.
+#' The order of the rows in the results is the same as the order in which the
+#' data was loaded, even if some rows fail (for example, due to missing
+#' values or unseen factor levels).
 #'
 #' @param object a fitted \linkS4class{H2OModel} object for which prediction is
 #'        desired
-#' @param newdata A \linkS4class{H2OFrame} object in which to look for
+#' @param newdata A Frame object in which to look for
 #'        variables with which to predict.
 #' @param ... additional arguments to pass on.
-#' @return Returns an \linkS4class{H2OFrame} object with probabilites and
+#' @return Returns an H2O Frame object with probabilites and
 #'         default predictions.
 #' @seealso \code{link{h2o.deeplearning}}, \code{link{h2o.gbm}},
 #'          \code{link{h2o.glm}}, \code{link{h2o.randomForest}} for model
@@ -372,15 +365,9 @@ predict.H2OModel <- function(object, newdata, ...) {
     stop("predictions with a missing `newdata` argument is not implemented yet")
   }
 
-  tmp_data <- !.is.eval(newdata)
-  if( tmp_data ) {
-    key  <- newdata@frame_id
-    .h2o.eval.frame(conn=h2o.getConnection(), ast=newdata@mutable$ast, frame_id=key)
-  }
-
   # Send keys to create predictions
-  url <- paste0('Predictions/models/', object@model_id, '/frames/', newdata@frame_id)
-  res <- .h2o.__remoteSend(object@conn, url, method = "POST")
+  url <- paste0('Predictions/models/', object@model_id, '/frames/',  h2o.getId(newdata))
+  res <- .h2o.__remoteSend(url, method = "POST")
   res <- res$predictions_frame
   h2o.getFrame(res$name)
 }
@@ -388,8 +375,7 @@ predict.H2OModel <- function(object, newdata, ...) {
 #' @export
 h2o.predict <- predict.H2OModel
 
-h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deeplearning"), params, strategy = c("mod1", "random"), ...)
-{
+h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deeplearning"), params, strategy = c("mod1", "random"), ...) {
   output <- data.frame()
 
   if( nfolds < 2 ) stop("`nfolds` must be greater than or equal to 2")
@@ -413,19 +399,18 @@ h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deepl
   # nfold_vec <- h2o.sample(fr, 1:nfolds)
   nfold_vec <- sample(rep(1:nfolds, length.out = data.len), data.len)
 
-  fnum_id <- as.h2o(nfold_vec, model@conn)
+  fnum_id <- as.h2o(nfold_vec)
   fnum_id <- h2o.cbind(fnum_id, data)
 
   xval <- lapply(1:nfolds, function(i) {
-      params$training_frame <- data[fnum_id$object != i, ]
-      params$validation_frame <- data[fnum_id$object != i, ]
+      params$training_frame   <- data[fnum_id[,1] != i, ]
+      params$validation_frame <- data[fnum_id[,1] == i, ]
       fold <- do.call(model.type, c(params))
       output[(i+1), "fold_num"] <<- i - 1
       output[(i+1), "model_key"] <<- fold@model_id
       # output[(i+1), "cv_err"] <<- mean(as.vector(fold@model$mse_valid))
       fold
     })
-  print(output)
 
   model
 }
@@ -437,7 +422,7 @@ h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deepl
 #'
 #'
 #' @param model An \linkS4class{H2OModel} object
-#' @param data An \linkS4class{H2OFrame}. The model will make predictions
+#' @param data An H2O Frame. The model will make predictions
 #'        on this dataset, and subsequently score them. The dataset should
 #'        match the dataset that was used to train the model, in terms of
 #'        column names, types, and dimensions. If data is passed in, then train and valid are ignored.
@@ -447,9 +432,9 @@ h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deepl
 #' @examples
 #' \donttest{
 #' library(h2o)
-#' localH2O <- h2o.init()
+#' h2o.init()
 #' prosPath <- system.file("extdata", "prostate.csv", package="h2o")
-#' prostate.hex <- h2o.uploadFile(localH2O, path = prosPath)
+#' prostate.hex <- h2o.uploadFile(path = prosPath)
 #' prostate.hex$CAPSULE <- as.factor(prostate.hex$CAPSULE)
 #' prostate.gbm <- h2o.gbm(3:9, "CAPSULE", prostate.hex)
 #' h2o.performance(model = prostate.gbm, data=prostate.hex)
@@ -458,39 +443,32 @@ h2o.crossValidate <- function(model, nfolds, model.type = c("gbm", "glm", "deepl
 h2o.performance <- function(model, data=NULL, valid=FALSE, ...) {
   # Some parameter checking
   if(!is(model, "H2OModel")) stop("`model` must an H2OModel object")
-  if(!is.null(data) && !is(data, "H2OFrame")) stop("`data` must be an H2OFrame object")
+  if(!is.null(data) && !is.Frame(data) ) stop("`data` must be an H2O Frame object")
 
   missingData <- missing(data) || is.null(data)
   trainingFrame <- model@parameters$training_frame
-  data.frame_id <- if( missingData ) trainingFrame else data@frame_id
-  if( !is.null(trainingFrame) && !missingData && data.frame_id == trainingFrame ) {
-    warning("Given data is same as the training data. Returning the training metrics.")
-    return(model@model$training_metrics)
-  }
-  else if( missingData && !valid ) return(model@model$training_metrics)    # no data, valid is false, return the training metrics
+  data.id <- if( missingData ) trainingFrame else h2o.getId(data)
+  if( missingData && !valid ) return(model@model$training_metrics)    # no data, valid is false, return the training metrics
   else if( missingData &&  valid ) {
     if( is.null(model@model$validation_metrics@metrics) ) return(NULL)
     else                                                  return(model@model$validation_metrics)  # no data, but valid is true, return the validation metrics
   }
   else if( !missingData ) {
-    mktmp <- !.is.eval(data)
-    if( mktmp ) .h2o.eval.frame(conn=h2o.getConnection(), ast=data@mutable$ast, frame_id=data@frame_id)
-
     parms <- list()
     parms[["model"]] <- model@model_id
-    parms[["frame"]] <- data.frame_id
-    res <- .h2o.__remoteSend(model@conn, method = "POST", .h2o.__MODEL_METRICS(model@model_id,data.frame_id), .params = parms)
+    parms[["frame"]] <- data.id
+    res <- .h2o.__remoteSend(method = "POST", .h2o.__MODEL_METRICS(model@model_id,data.id), .params = parms)
 
     ####
     # FIXME need to do the client-side filtering...  PUBDEV-874:   https://0xdata.atlassian.net/browse/PUBDEV-874
-    model_metrics <- Filter(function(mm) { mm$frame$name==data.frame_id}, res$model_metrics)[[1]]   # filter on data.frame_id, R's builtin Filter function
+    model_metrics <- Filter(function(mm) { mm$frame$name==data.id}, res$model_metrics)[[1]]   # filter on data.id, R's builtin Filter function
     #
     ####
     metrics <- model_metrics[!(names(model_metrics) %in% c("__meta", "names", "domains", "model_category"))]
     model_category <- model_metrics$model_category
     Class <- paste0("H2O", model_category, "Metrics")
     metrics$frame <- list()
-    metrics$frame$name <- data.frame_id
+    metrics$frame$name <- data.id
     new(Class     = Class,
         algorithm = model@algorithm,
         on_train  = missingData,
@@ -621,7 +599,7 @@ h2o.aic <- function(object, train=FALSE, valid=FALSE, xval=FALSE, ...) {
 #'
 #' @param object An \linkS4class{H2OModel} object.
 #' @param train Retrieve the training R2
-#' @param valid Retrieve the validation R2
+#' @param valid  Retrieve the validation set R2 if a validation set was passed in during model build time.
 #' @param xval Retrieve the cross-validation R2
 #' @param \dots extra arguments to be passed if `object` is of type
 #'              \linkS4class{H2OModel} (e.g. train=TRUE)
@@ -995,6 +973,7 @@ h2o.weights <- function(object, matrix_id=1, ...){
     warning( paste0("No weights for ", class(o)) )
     return(NULL)
   }
+  h2o.getFrame(sh$name)
 }
 
 #'
@@ -1014,6 +993,7 @@ h2o.biases <- function(object, vector_id=1, ...){
     warning( paste0("No biases for ", class(o)) )
     return(NULL)
   }
+  h2o.getFrame(sh$name)
 }
 
 #'
@@ -1225,16 +1205,24 @@ h2o.specificity <- function(object, thresholds){
   h2o.metric(object, thresholds, "tnr")
 }
 
-#
-#
+#' Find the threshold, give the max metric
+#'
+#' @rdname h2o.find_threshold_by_max_metric
+#' @param object H2OBinomialMetrics
+#' @param metric "F1," for example
+#' @export
 h2o.find_threshold_by_max_metric <- function(object, metric) {
   if(!is(object, "H2OBinomialMetrics")) stop(paste0("No ", metric, " for ",class(object)))
   max_metrics <- object@metrics$max_criteria_and_metric_scores
   max_metrics[match(paste0("max ",metric),max_metrics$metric),"threshold"]
 }
 
-#
-# No duplicate thresholds allowed
+#' Find the threshold, give the max metric. No duplicate thresholds allowed
+#'
+#' @rdname h2o.find_row_by_threshold
+#' @param object H2OBinomialMetrics
+#' @param threshold number between 0 and 1
+#' @export
 h2o.find_row_by_threshold <- function(object, threshold) {
   if(!is(object, "H2OBinomialMetrics")) stop(paste0("No ", threshold, " for ",class(object)))
   tmp <- object@metrics$thresholds_and_metric_scores
@@ -1655,7 +1643,7 @@ h2o.null_dof <- function(object, train=FALSE, valid=FALSE, xval=FALSE, ...) {
 #'
 #' @param object Either an \linkS4class{H2OModel} object or an
 #'        \linkS4class{H2OModelMetrics} object.
-#' @param newdata An \linkS4class{H2OFrame} object that can be scored on.
+#' @param newdata An H2O Frame object that can be scored on.
 #'        Requires a valid response column.
 #' @param thresholds (Optional) A value or a list of valid values between 0.0 and 1.0.
 #'        This value is only used in the case of
@@ -1700,14 +1688,8 @@ setMethod("h2o.confusionMatrix", "H2OModel", function(object, newdata, valid=FAL
   } else if( valid ) stop("Cannot have both `newdata` and `valid=TRUE`", call.=FALSE)
 
   # ok need to score on the newdata
-  tmp <- !.is.eval(newdata)
-  if( tmp ) {
-    temp_key <- newdata@frame_id
-    .h2o.eval.frame(conn = newdata@conn, ast = newdata@mutable$ast, frame_id = temp_key)
-  }
-
-  url <- paste0("Predictions/models/",object@model_id, "/frames/", newdata@frame_id)
-  res <- .h2o.__remoteSend(object@conn, url, method="POST")
+  url <- paste0("Predictions/models/",object@model_id, "/frames/", h2o.getId(newdata))
+  res <- .h2o.__remoteSend(url, method="POST")
 
   # Make the correct class of metrics object
   metrics <- new(sub("Model", "Metrics", class(object)), algorithm=object@algorithm, metrics= res$model_metrics[[1L]])   # FIXME: don't think model metrics come out of Predictions anymore!!!
@@ -1785,6 +1767,41 @@ setMethod("h2o.confusionMatrix", "H2OModelMetrics", function(object, thresholds=
   m
 })
 
+#' Plot an H2O Model
+#'
+#' Plots training set (and validation set if available) scoring history for an H2O Model
+#'
+#' This method dispatches on the type of H2O model to select the correct
+#' scoring history.  The \code{timestep} and \code{metric} arguments are restricted to what is
+#' available in the scoring history for a particular type of model.
+#'
+#' @param x A fitted \linkS4class{H2OModel} object for which the scoring history plot is desired.
+#' @param timestep A unit of measurement for the x-axis.
+#' @param metric A unit of measurement for the y-axis.
+#' @param ... additional arguments to pass on.
+#' @return Returns a scoring history plot.
+#' @seealso \code{link{h2o.deeplearning}}, \code{link{h2o.gbm}},
+#'          \code{link{h2o.glm}}, \code{link{h2o.randomForest}} for model
+#'          generation in h2o.
+#' @examples
+#' \donttest{
+#' library(h2o)
+#' library(mlbench)
+#' h2o.init()
+#'
+#' df <- as.h2o(mlbench::mlbench.friedman1(10000,1))
+#' rng <- h2o.runif(df, seed=1234)
+#' train <- df[rng<0.8,]
+#' valid <- df[rng>=0.8,]
+#'
+#' gbm <- h2o.gbm(x = 1:10, y = "y", training_frame = train, validation_frame = valid,
+#'   ntrees=500, learn_rate=0.01, score_each_iteration = TRUE)
+#' plot(gbm)
+#' plot(gbm, timestep = "duration", metric = "deviance")
+#' plot(gbm, timestep = "number_of_trees", metric = "deviance")
+#' plot(gbm, timestep = "number_of_trees", metric = "MSE")
+#'
+#' }
 #' @export
 plot.H2OModel <- function(x, timestep = "AUTO", metric = "AUTO", ...) {
   df <- as.data.frame(x@model$scoring_history)
@@ -1799,25 +1816,33 @@ plot.H2OModel <- function(x, timestep = "AUTO", metric = "AUTO", ...) {
       stop("for GLM, metric must be one of: log_likelihood, objective")
     }
     graphics::plot(df$iteration, df[,c(metric)], type="l", xlab = timestep, ylab = metric, main = "Validation Scoring History")
+  } else if (x@algorithm == "glrm") {
+    timestep <- "iteration"
+    if (metric == "AUTO") {
+      metric <- "objective"
+    } else if (!(metric %in% c("step_size", "objective"))) {
+      stop("for GLRM, metric must be one of: step_size, objective")
+    }
+    graphics::plot(df$iteration, df[,c(metric)], type="l", xlab = timestep, ylab = metric, main = "Objective Function Value per Iteration")
   } else if (x@algorithm %in% c("deeplearning", "drf", "gbm")) {
     if (is(x, "H2OBinomialModel")) {
       if (metric == "AUTO") {
         metric <- "logloss"
-      } else if (!(metric %in% c("r2","logloss","AUC","classification_error","MSE"))) {
-        stop("metric for H2OBinomialModel must be one of: AUTO, r2, logloss, AUC, classification_error, MSE")
-      } 
+      } else if (!(metric %in% c("logloss","AUC","classification_error","MSE"))) {
+        stop("metric for H2OBinomialModel must be one of: AUTO, logloss, AUC, classification_error, MSE")
+      }
     } else if (is(x, "H2OMultinomialModel")) {
       if (metric == "AUTO") {
         metric <- "classification_error"
-      } else if (!(metric %in% c("r2","logloss","classification_error","MSE"))) {
-        stop("metric for H2OMultinomialModel must be one of: AUTO, r2, logloss, classification_error, MSE")
-      } 
+      } else if (!(metric %in% c("logloss","classification_error","MSE"))) {
+        stop("metric for H2OMultinomialModel must be one of: AUTO, logloss, classification_error, MSE")
+      }
     } else if (is(x, "H2ORegressionModel")) {
       if (metric == "AUTO") {
         metric <- "MSE"
       } else if (!(metric %in% c("MSE","deviance"))) {
-        stop("metric for H2OMultinomialModel must be one of: MSE, deviance")
-      } 
+        stop("metric for H2ORegressionModel must be one of: AUTO, MSE, deviance")
+      }
     } else {
       stop("Must be one of: H2OBinomialModel, H2OMultinomialModel or H2ORegressionModel")
     }
@@ -1827,8 +1852,8 @@ plot.H2OModel <- function(x, timestep = "AUTO", metric = "AUTO", ...) {
         timestep <- "number_of_trees"
       } else if (!(timestep %in% c("duration","number_of_trees"))) {
         stop("timestep for gbm or drf must be one of: duration, number_of_trees")
-      } 
-    } else if (x@algorithm == "deeplearning") {
+      }
+    } else { # x@algorithm == "deeplearning"
       # Delete first row of DL scoring history since it contains NAs & NaNs
       if (df$samples[1] == 0) {
         df <- df[-1,]
@@ -1837,36 +1862,46 @@ plot.H2OModel <- function(x, timestep = "AUTO", metric = "AUTO", ...) {
         timestep <- "epochs"
       } else if (!(timestep %in% c("epochs","samples","duration"))) {
         stop("timestep for deeplearning must be one of: epochs, samples, duration")
-      } 
-    } else {
-      stop("Plotting not implemented for this type of model")
+      }
     }
     training_metric <- sprintf("training_%s", metric)
     validation_metric <- sprintf("validation_%s", metric)
     if (timestep == "duration") {
-      tt <- trimws(df[2, c("duration")])
+      trim <- function (ss) gsub("^\\s+|\\s+$", "", ss)
+      tt <- trim(df[2, c("duration")])  #base::trimws not implemented for earlier versions of R, so we make our own trim function
       dur_colname <- sprintf("duration_%s", strsplit(tt, " ")[[1]][2]) #parse units of measurement
-      df[,c(dur_colname)] <- apply(as.matrix(df[,c("duration")]), 1, function(v) as.numeric(strsplit(trimws(v), " ")[[1]][1]))
+      df[,c(dur_colname)] <- apply(as.matrix(df[,c("duration")]), 1, function(v) as.numeric(strsplit(trim(v), " ")[[1]][1]))
       timestep <- dur_colname
     }
-    ylim <- range(c(df[,c(training_metric)], df[,c(validation_metric)]))  #sync up y axes
-    graphics::plot(df[,c(timestep)], df[,c(training_metric)], type="l", xlab = "", ylab = "", axes = FALSE,
-         main = "Scoring History", col = "blue", ylim = ylim)
-    graphics::par(new = TRUE)
-    graphics::plot(df[,c(timestep)], df[,c(validation_metric)], type="l", xlab = timestep, ylab = metric, col = "orange", ylim = ylim)
-    graphics::legend("topright", legend = c("Training", "Validation"), col = c("blue", "orange"), lty = c(1,1))
+    if (validation_metric %in% names(df)) {  #Training and Validation scoring history
+      ylim <- range(c(df[,c(training_metric)], df[,c(validation_metric)]))  #sync up y axes
+      graphics::plot(df[,c(timestep)], df[,c(training_metric)], type="l", xlab = "", ylab = "", axes = FALSE,
+                     main = "Scoring History", col = "blue", ylim = ylim)
+      graphics::par(new = TRUE)
+      graphics::plot(df[,c(timestep)], df[,c(validation_metric)], type="l", xlab = timestep, ylab = metric, col = "orange", ylim = ylim)
+      graphics::legend("topright", legend = c("Training", "Validation"), col = c("blue", "orange"), lty = c(1,1))
+    } else {  #Training scoring history only
+      ylim <- range(c(df[,c(training_metric)]))
+      graphics::plot(df[,c(timestep)], df[,c(training_metric)], type="l", xlab = timestep, ylab = training_metric,
+                     main = "Training Scoring History", col = "blue", ylim = ylim)
+
+    }
+  } else { # algo is not glm, deeplearning, drf, gbm
+  	stop("Plotting not implemented for this type of model")
   }
 }
 
 #' @export
-plot.H2OBinomialMetrics <- function(x, type = "roc", ...) {
+plot.H2OBinomialMetrics <- function(x, type = "roc", main, ...) {
   # TODO: add more types (i.e. cutoffs)
   if(!type %in% c("roc")) stop("type must be 'roc'")
   if(type == "roc") {
     xaxis <- "False Positive Rate"; yaxis = "True Positive Rate"
-    main <- paste(yaxis, "vs", xaxis)
-    if( x@on_train ) main <- paste(main, "(on train)")
-    else             main <- paste(main, "(on valid)")
+    if(missing(main)) {
+      main <- paste(yaxis, "vs", xaxis)
+      if( x@on_train ) main <- paste(main, "(on train)")
+      else             main <- paste(main, "(on valid)")
+    }
     graphics::plot(x@metrics$thresholds_and_metric_scores$fpr, x@metrics$thresholds_and_metric_scores$tpr, main = main, xlab = xaxis, ylab = yaxis, ylim=c(0,1), xlim=c(0,1), ...)
     graphics::abline(0, 1, lty = 2)
   }
@@ -1902,15 +1937,6 @@ h2o.sdev <- function(object) {
   as.numeric(object@model$importance[1,])
 }
 
-# Handles ellipses
-.model.ellipses <- function(dots) {
-  lapply(names(dots), function(type) {
-    stop(paste0('\n  unexpected argument "',
-                type,'", is this legacy code? Try ?h2o.shim'), call. = FALSE)
-  })
-}
-
-
 # extract "bite size" pieces from a model
 .model.parts <- function(object) {
   o  <- object
@@ -1918,9 +1944,9 @@ h2o.sdev <- function(object) {
   tm <- object@model$training_metrics
   vm <- object@model$validation_metrics
   xm <- object@model$cross_validation_metrics
-  if( !is.null(vm@metrics) && !is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=vm,xm=xm) )
-  if( is.null(vm@metrics) && !is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=NULL,xm=xm) )
-  if( !is.null(vm@metrics) && is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=vm,xm=NULL) )
+  if( !is.null(vm@metrics) && !is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=  vm,xm=  xm) )
+  if(  is.null(vm@metrics) && !is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=NULL,xm=  xm) )
+  if( !is.null(vm@metrics) &&  is.null(xm@metrics) ) return( list(o=o,m=m,tm=tm,vm=  vm,xm=NULL) )
   return( list(o=o,m=m,tm=tm,vm=NULL,xm=NULL) )
 }
 
@@ -1953,13 +1979,21 @@ h2o.sdev <- function(object) {
   result
 }
 
+#
+# Returns REST API version for given algo.
+#
+.h2o.getAlgoVersion <- function(algo, h2oRestApiVersion = .h2o.__REST_API_VERSION) {
+  result <- .h2o.__remoteSend(method = "GET", h2oRestApiVersion = h2oRestApiVersion, .h2o.__MODEL_BUILDERS(algo))$model_builders[[algo]][["__meta"]]$schema_version
+  result
+}
+
 #' Tabulation between Two Columns of a H2O Frame
 #'
 #' Simple Co-Occurrence based tabulation of X vs Y, where X and Y are two Vecs in a given dataset.
 #' Uses histogram of given resolution in X and Y.
 #' Handles numerical/categorical data and missing values. Supports observation weights.
 #'
-#' @param data An \linkS4class{H2OFrame} object.
+#' @param data An H2O Frame object.
 #' @param x predictor column
 #' @param y response column
 #' @param weights_column (optional) observation weights column
@@ -1971,10 +2005,11 @@ h2o.sdev <- function(object) {
 #' @examples
 #' \donttest{
 #' library(h2o)
-#' localH2O <- h2o.init()
+#' h2o.init()
 #' df <- as.h2o(iris)
-#' h2o.tabulate(data = df, x = "Sepal.Length", y = "Petal.Width",
+#' tab <- h2o.tabulate(data = df, x = "Sepal.Length", y = "Petal.Width",
 #'              weights_column = NULL, nbins_x = 10, nbins_y = 10)
+#' plot(tab)              
 #' }
 #' @export
 h2o.tabulate <- function(data, x, y,
@@ -1982,28 +2017,82 @@ h2o.tabulate <- function(data, x, y,
                          nbins_x = 50,
                          nbins_y = 50
                          ) {
-  delete <- !.is.eval(data)
-  if (delete) {
-    temp_key <- data@frame_id
-    .h2o.eval.frame(conn = data@conn, ast = data@mutable$ast, frame_id = temp_key)
-  }
   args <- .verify_datacols(data, c(x,y))
   if(!is.numeric(nbins_x)) stop("`nbins_x` must be a positive number")
   if(!is.numeric(nbins_y)) stop("`nbins_y` must be a positive number")
 
   parms = list()
-  parms$dataset <- data@frame_id
+  parms$dataset <- attr(data, "id")
   parms$predictor <- args$cols[1]
   parms$response <- args$cols[2]
   if( !missing(weights_column) )            parms$weight <- weights_column
   parms$nbins_predictor <- nbins_x
   parms$nbins_response <- nbins_y
 
-  res <- .h2o.__remoteSend(conn = data@conn, method = "POST", h2oRestApiVersion = 99, page = "Tabulate", .params = parms)
-  print(res)
+  res <- .h2o.__remoteSend(method = "POST", h2oRestApiVersion = 99, page = "Tabulate", .params = parms)
   count_table <- res$count_table
   response_table <- res$response_table
   out <- list(count_table = count_table, response_table = response_table, cols = args$cols)
   oldClass(out) <- c("H2OTabulate", "list")
   out
 }
+
+#' Plot an H2O Tabulate Heatmap
+#'
+#' Plots the simple co-occurrence based tabulation of X vs Y as a heatmap, where X and Y are two Vecs in a given dataset.
+#'
+#' @param x An H2OTabulate object for which the heatmap plot is desired.
+#' @param xlab A title for the x-axis.  Defaults to what is specified in the given H2OTabulate object.
+#' @param ylab A title for the y-axis.  Defaults to what is specified in the given H2OTabulate object.
+#' @param base_size  Base font size for plot.
+#' @param ... additional arguments to pass on.
+#' @return Returns a ggplot2-based heatmap of co-occurance.
+#' @seealso \code{link{h2o.tabulate}}
+#' @examples
+#' \donttest{
+#' library(h2o)
+#' h2o.init()
+#' df <- as.h2o(iris)
+#' tab <- h2o.tabulate(data = df, x = "Sepal.Length", y = "Petal.Width",
+#'              weights_column = NULL, nbins_x = 10, nbins_y = 10)
+#' plot(tab)              
+#' }
+#' @export
+plot.H2OTabulate <- function(x, xlab = x$cols[1], ylab = x$cols[2], base_size = 12, ...) {
+  
+  if (!inherits(x, "H2OTabulate")) {
+    stop("Must be an H2OTabulate object")
+  }
+  
+  # Pull small counts table into R memory to plot
+  df <- as.data.frame(x$count_table)
+  names(df) <- c("c1", "c2", "counts")
+  
+  # Reorder the levels for better plotting
+  if (suppressWarnings(is.na(sum(as.numeric(df$c1))))) {
+    c1_order <- order(unique(df$c1))
+  } else {
+    c1_order <- order(unique(as.numeric(df$c1)))
+  }
+  if (suppressWarnings(is.na(sum(as.numeric(df$c2))))) {
+    c2_order <- order(unique(df$c2))
+  } else {
+    c2_order <- order(unique(as.numeric(df$c2)))
+  }
+  c1_labels <- unique(df$c1)
+  c2_labels <- unique(df$c2)
+  df$c1 <- factor(df$c1, levels = c1_labels[c1_order])
+  df$c2 <- factor(df$c2, levels = c2_labels[c2_order])
+  
+  # Plot heatmap
+  c1 <- c2 <- counts <- NULL #set these to pass CRAN checks w/o warnings
+  (p <- ggplot2::ggplot(df, ggplot2::aes(c1, c2)) 
+  + ggplot2::geom_tile(ggplot2::aes(fill = counts), colour = "white") + ggplot2::scale_fill_gradient(low = "white", high = "steelblue"))
+  
+  # Adjust the plot
+  p <- p + ggplot2::theme_grey(base_size = base_size) + ggplot2::labs(x = xlab, y = ylab) + ggplot2::scale_x_discrete(expand = c(0, 0)) + ggplot2::scale_y_discrete(expand = c(0, 0)) + ggplot2::theme(legend.position = "none", axis.ticks = ggplot2::element_blank(), axis.text.x = ggplot2::element_text(size = base_size * 0.8, angle = 330, hjust = 0, colour = "grey50"))
+  
+  # Return a ggplot object
+  return(p)
+}
+
