@@ -14,7 +14,7 @@
 #' @param y The name or index of the response variable. If the data does not contain a header, this is the
 #'        column index number starting at 0, and increasing from left to right. The response must be a categorical
 #'        variable with at least two levels.
-#' @param training_frame An \code{\linkS4class{H2OFrame}} object containing the variables in the model.
+#' @param training_frame An H2O Frame object containing the variables in the model.
 #' @param model_id (Optional) The unique id assigned to the resulting model. If
 #'        none is given, an id will automatically be generated.
 #' @param laplace A positive number controlling Laplace smoothing. The default zero disables smoothing.
@@ -29,13 +29,13 @@
 #'        every row in the training dataset that contains at least one NA will be skipped completely.
 #'        If the test dataset has missing values, then those predictors are omitted in the probability
 #'        calculation during prediction.
-#' @return Returns an object of class \linkS4class{H2OBinomialModel} if the response has two categorical levels, 
+#' @return Returns an object of class \linkS4class{H2OBinomialModel} if the response has two categorical levels,
 #'         and \linkS4class{H2OMultinomialModel} otherwise.
 #' @examples
 #' \donttest{
-#'  localH2O <- h2o.init()
+#'  h2o.init()
 #'  votesPath <- system.file("extdata", "housevotes.csv", package="h2o")
-#'  votes.hex <- h2o.uploadFile(localH2O, path = votesPath, header = TRUE)
+#'  votes.hex <- h2o.uploadFile(path = votesPath, header = TRUE)
 #'  h2o.naiveBayes(x = 2:17, y = 1, training_frame = votes.hex, laplace = 3)
 #' }
 #' @export
@@ -46,11 +46,11 @@ h2o.naiveBayes <- function(x, y, training_frame,
                            eps = 0,
                            compute_metrics = TRUE)
 {
-  # Training_frame may be a key or an H2OFrame object
-  if (!inherits(training_frame, "H2OFrame"))
+  # Training_frame may be a key or an H2O Frame object
+  if (!is.Frame(training_frame))
     tryCatch(training_frame <- h2o.getFrame(training_frame),
              error = function(err) {
-               stop("argument \"training_frame\" must be a valid H2OFrame or key")
+               stop("argument \"training_frame\" must be a valid Frame or key")
              })
 
   .naivebayes.map <- c("x" = "ignored_columns", "y" = "response_column",
@@ -61,6 +61,7 @@ h2o.naiveBayes <- function(x, y, training_frame,
   args <- .verify_dataxy(training_frame, x, y)
   parms$ignored_columns <- args$x_ignore
   parms$response_column <- args$y
+  parms$training_frame <- training_frame
   if(!missing(model_id))
     parms$model_id <- model_id
   if(!missing(laplace))
@@ -78,5 +79,5 @@ h2o.naiveBayes <- function(x, y, training_frame,
   parms$eps_prob <- eps
 
   # Error check and build model
-  .h2o.createModel(training_frame@conn, 'naivebayes', parms)
+  .h2o.modelJob('naivebayes', parms)
 }
